"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const common_tags_1 = require("common-tags");
const from_fixture_1 = require("./from-fixture");
describe("fromFixture", () => {
    it("should create an invalid test with a message ID", () => {
        const test = (0, from_fixture_1.fromFixture)((0, common_tags_1.stripIndent) `
        const name = "alice";
                     ~~~~~~~ [whoops]
      `);
        (0, chai_1.expect)(test).to.have.property("code", `const name = "alice";`);
        (0, chai_1.expect)(test).to.have.property("errors");
        (0, chai_1.expect)(test.errors).to.deep.equal([
            {
                column: 14,
                data: {},
                endColumn: 21,
                endLine: 1,
                line: 1,
                messageId: "whoops",
            },
        ]);
    });
    it("should create an invalid test with options", () => {
        const test = (0, from_fixture_1.fromFixture)((0, common_tags_1.stripIndent) `
        const name = "alice";
                     ~~~~~~~ [whoops]
      `, {
            filename: "test.ts",
            output: (0, common_tags_1.stripIndent) `
          const name = 'alice';
        `,
        });
        (0, chai_1.expect)(test).to.have.property("code", `const name = "alice";`);
        (0, chai_1.expect)(test).to.have.property("errors");
        (0, chai_1.expect)(test.errors).to.deep.equal([
            {
                column: 14,
                data: {},
                endColumn: 21,
                endLine: 1,
                line: 1,
                messageId: "whoops",
            },
        ]);
        (0, chai_1.expect)(test).to.have.property("filename", "test.ts");
        (0, chai_1.expect)(test).to.have.property("output", "const name = 'alice';");
    });
    it("should create an invalid test with multiple errors", () => {
        const test = (0, from_fixture_1.fromFixture)((0, common_tags_1.stripIndent) `
        const name = "alice";
                     ~~~~~~~ [first]
              ~~~~ [second]
        const role = 'engineer';
        ~~~~~ [third]
      `);
        (0, chai_1.expect)(test).to.have.property("code", (0, common_tags_1.stripIndent) `
      const name = "alice";
      const role = 'engineer';
    `);
        (0, chai_1.expect)(test).to.have.property("errors");
        (0, chai_1.expect)(test.errors).to.deep.equal([
            {
                column: 14,
                data: {},
                endColumn: 21,
                endLine: 1,
                line: 1,
                messageId: "first",
            },
            {
                column: 7,
                data: {},
                endColumn: 11,
                endLine: 1,
                line: 1,
                messageId: "second",
            },
            {
                column: 1,
                data: {},
                endColumn: 6,
                endLine: 2,
                line: 2,
                messageId: "third",
            },
        ]);
    });
    it("should create an invalid test with data", () => {
        const test = (0, from_fixture_1.fromFixture)((0, common_tags_1.stripIndent) `
        const name = "alice";
              ~~~~ [whoops { "identifier": "name" }]
      `);
        (0, chai_1.expect)(test).to.have.property("code", `const name = "alice";`);
        (0, chai_1.expect)(test).to.have.property("errors");
        (0, chai_1.expect)(test.errors).to.deep.equal([
            {
                column: 7,
                data: {
                    identifier: "name",
                },
                endColumn: 11,
                endLine: 1,
                line: 1,
                messageId: "whoops",
            },
        ]);
    });
    it("should support data that contains punctuation", () => {
        const punctuation = "[]{}()";
        const test = (0, from_fixture_1.fromFixture)((0, common_tags_1.stripIndent) `
        const name = "alice";
              ~~~~ [whoops { "value": "${punctuation}" }]
      `);
        (0, chai_1.expect)(test).to.have.property("code", `const name = "alice";`);
        (0, chai_1.expect)(test).to.have.property("errors");
        (0, chai_1.expect)(test.errors).to.deep.equal([
            {
                column: 7,
                data: {
                    value: punctuation,
                },
                endColumn: 11,
                endLine: 1,
                line: 1,
                messageId: "whoops",
            },
        ]);
    });
    it("should create an invalid test with a suggestion", () => {
        const test = (0, from_fixture_1.fromFixture)((0, common_tags_1.stripIndent) `
        const name = "alice";
                     ~~~~~~~ [whoops suggest]
      `, {
            suggestions: [
                {
                    messageId: "wat",
                    output: (0, common_tags_1.stripIndent) `
              const name = "bob";
            `,
                },
            ],
        });
        (0, chai_1.expect)(test).to.have.property("code", `const name = "alice";`);
        (0, chai_1.expect)(test).to.have.property("errors");
        (0, chai_1.expect)(test.errors).to.deep.equal([
            {
                column: 14,
                data: {},
                endColumn: 21,
                endLine: 1,
                line: 1,
                messageId: "whoops",
                suggestions: [
                    {
                        messageId: "wat",
                        output: (0, common_tags_1.stripIndent) `
              const name = "bob";
            `,
                    },
                ],
            },
        ]);
        (0, chai_1.expect)(test).to.not.have.property("suggestions");
    });
    it("should create an invalid test with multiple suggestions", () => {
        const test = (0, from_fixture_1.fromFixture)((0, common_tags_1.stripIndent) `
        const name = "alice";
                     ~~~~~~~ [whoops suggest]
      `, {
            suggestions: [
                {
                    messageId: "wat",
                    output: (0, common_tags_1.stripIndent) `
              const name = "bob";
            `,
                },
                {
                    messageId: "wat",
                    output: (0, common_tags_1.stripIndent) `
              const name = "eve";
            `,
                },
            ],
        });
        (0, chai_1.expect)(test).to.have.property("code", `const name = "alice";`);
        (0, chai_1.expect)(test).to.have.property("errors");
        (0, chai_1.expect)(test.errors).to.deep.equal([
            {
                column: 14,
                data: {},
                endColumn: 21,
                endLine: 1,
                line: 1,
                messageId: "whoops",
                suggestions: [
                    {
                        messageId: "wat",
                        output: (0, common_tags_1.stripIndent) `
              const name = "bob";
            `,
                    },
                    {
                        messageId: "wat",
                        output: (0, common_tags_1.stripIndent) `
              const name = "eve";
            `,
                    },
                ],
            },
        ]);
        (0, chai_1.expect)(test).to.not.have.property("suggestions");
    });
    it("should create an invalid test with multiple errors with suggestions", () => {
        const test = (0, from_fixture_1.fromFixture)((0, common_tags_1.stripIndent) `
        const name = "alice";
                     ~~~~~~~ [whoops suggest 0]
                     ~~~~~~~ [whoops { "identifier": "name" } suggest 1 2]
                     ~~~~~~~ [whoops]
      `, {
            suggestions: [
                {
                    messageId: "wat",
                    output: (0, common_tags_1.stripIndent) `
              const name = "bob";
            `,
                },
                {
                    messageId: "wat",
                    output: (0, common_tags_1.stripIndent) `
              const name = "eve";
            `,
                },
                {
                    messageId: "wat",
                    output: (0, common_tags_1.stripIndent) `
              const name = "mallory";
            `,
                },
            ],
        });
        (0, chai_1.expect)(test).to.have.property("code", `const name = "alice";`);
        (0, chai_1.expect)(test).to.have.property("errors");
        (0, chai_1.expect)(test.errors).to.deep.equal([
            {
                column: 14,
                data: {},
                endColumn: 21,
                endLine: 1,
                line: 1,
                messageId: "whoops",
                suggestions: [
                    {
                        messageId: "wat",
                        output: (0, common_tags_1.stripIndent) `
              const name = "bob";
            `,
                    },
                ],
            },
            {
                column: 14,
                data: { identifier: "name" },
                endColumn: 21,
                endLine: 1,
                line: 1,
                messageId: "whoops",
                suggestions: [
                    {
                        messageId: "wat",
                        output: (0, common_tags_1.stripIndent) `
              const name = "eve";
            `,
                    },
                    {
                        messageId: "wat",
                        output: (0, common_tags_1.stripIndent) `
              const name = "mallory";
            `,
                    },
                ],
            },
            {
                column: 14,
                data: {},
                endColumn: 21,
                endLine: 1,
                line: 1,
                messageId: "whoops",
            },
        ]);
        (0, chai_1.expect)(test).to.not.have.property("suggestions");
    });
    it("should throw if suggestions are not annotated", () => {
        (0, chai_1.expect)(() => (0, from_fixture_1.fromFixture)((0, common_tags_1.stripIndent) `
        const name = "alice";
                     ~~~~~~~ [whoops]
      `, {
            suggestions: [
                {
                    messageId: "wat",
                    output: (0, common_tags_1.stripIndent) `
              const name = "bob";
            `,
                },
            ],
        })).to.throw(/no 'suggest' annotation found/);
    });
});
